var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import * as React from "react";
import createEmotionServer from "@emotion/server/create-instance";
import { CacheProvider as DefaultCacheProvider } from "@emotion/react";
import createCache from "@emotion/cache";
import DefaultDocument from "next/document";
/** @see <https://docs.tss-react.dev/ssr/next> */
export function createEmotionSsrAdvancedApproach(
/** This is the options passed to createCache() from 'import createCache from "@emotion/cache"' */
options, 
/** By default <CacheProvider /> from 'import { CacheProvider } from "@emotion/react"' */
CacheProvider = DefaultCacheProvider) {
    const { prepend } = options, optionsWithoutPrependProp = __rest(options, ["prepend"]);
    const propName = `${options.key}EmotionCache`;
    function augmentDocumentWithEmotionCache(Document) {
        var _a, _b;
        const super_getInitialProps = (_b = (_a = Document.getInitialProps) === null || _a === void 0 ? void 0 : _a.bind(Document)) !== null && _b !== void 0 ? _b : DefaultDocument.getInitialProps.bind(DefaultDocument);
        Document.getInitialProps = (appContext) => __awaiter(this, void 0, void 0, function* () {
            const cache = createCache(optionsWithoutPrependProp);
            const emotionServer = createEmotionServer(cache);
            const originalRenderPage = appContext.renderPage;
            appContext.renderPage = () => originalRenderPage({
                "enhanceApp": (App) => function EnhanceApp(props) {
                    return React.createElement(App, Object.assign({}, Object.assign(Object.assign({}, props), { [propName]: cache })));
                },
            });
            const initialProps = yield super_getInitialProps(appContext);
            const emotionStyles = emotionServer
                .extractCriticalToChunks(initialProps.html)
                .styles.filter(({ css }) => css !== "")
                .map(style => (React.createElement("style", { "data-emotion": `${style.key} ${style.ids.join(" ")}`, key: style.key, dangerouslySetInnerHTML: { "__html": style.css } })));
            const otherStyles = React.Children.toArray(initialProps.styles);
            return Object.assign(Object.assign({}, initialProps), { "styles": prepend
                    ? [...emotionStyles, ...otherStyles]
                    : [...otherStyles, ...emotionStyles] });
        });
    }
    function withAppEmotionCache(App) {
        function AppWithEmotionCache(props) {
            const _a = props, _b = propName, cache = _a[_b], rest = __rest(_a, [typeof _b === "symbol" ? _b : _b + ""]);
            return (React.createElement(CacheProvider, { value: cache !== null && cache !== void 0 ? cache : createCache(optionsWithoutPrependProp) },
                React.createElement(App, Object.assign({}, rest))));
        }
        Object.keys(App).forEach(staticMethod => (AppWithEmotionCache[staticMethod] = App[staticMethod]));
        AppWithEmotionCache.displayName = AppWithEmotionCache.name;
        return AppWithEmotionCache;
    }
    return { withAppEmotionCache, augmentDocumentWithEmotionCache };
}
